@@ -15,8 +15,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *----------------------------------------------------------------------------
+ * Version 1.9
+ *   - Correct calculation of timecodes. Integer only, now.
+ *   - Remove support for drop timecodes. If anybody really needs them, and
+ *     has a way to check calculated timecodes are correct, please tell me.
+ *
  * Version 1.8
- *   - Fix crash with certain frame size (unaligned buffer used with SSE2)
+ *   - Fix crash with certain frame sizes (unaligned buffer used with SSE2)
  *
  * Version 1.7
  *   - Terminate using exit/return instead of abort (no more "crashes")
@@ -510,22 +515,20 @@ void swap_rb (stream_info_t *s_info, char *img, char *out)
 		swap_rb_c(s_info, img, out);
 }
 
-/* SMPTE time code */
-void mk_timecode (int frame, double fps, char *buf) /* buf must have length 12 (incl. trailing \0) */
+/* SMPTE non-drop time code */
+void mk_timecode (int frame, int fps, char *buf) /* buf must have length 12 (incl. trailing \0) */
 {
 	int frames, s, m, h;
-	double tc = frame / fps;
-	double ms;
-	double tc_clean;
-	s = (int)tc % 60;
-	tc /= 60.0;
-	m = (int)tc % 60;
-	tc /= 60.0;
-	h = (int)tc;
+	int tc = frame;
 
-	tc_clean = frame / fps - (h * 60 * 60 + m * 60 + s);
-	tc_clean *= fps;
-	frames = (int)tc_clean;
+	tc = frame;
+	frames = tc % fps;
+	tc /= fps;
+	s = tc % 60;
+	tc /= 60;
+	m = tc % 60;
+	tc /= 60;
+	h = tc;
 
 	if (h > 99)
 	{
@@ -548,7 +551,7 @@ void print_usage ()
 		"LANGUAGE\tLanguage code, like: und\n"
 		"VIDEOFORMAT\tEither of: 480i, 480p, 576i, 720p, 1080i, 1080p\n"
 		"FRAMERATE\tEither of: 23.976, 24, 25, 29.97, 50, 59.94\n"
-		"         \tFor drop timecodes, use: 23.976d, 29.97d, 59.94d\n"
+		/*"         \tFor drop timecodes, use: 23.976d, 29.97d, 59.94d\n"*/
 		"OUTPUT\t\tOutput file in BDN XML format\n\n"
 		"Example: avs2bdnxml input.avs Undefined und 1080p 23.976 output.xml\n");
 }
@@ -587,19 +590,29 @@ struct framerate_entry_s
 {
 	char *name;
 	char *out_name;
-	double rate;
+	int rate;
 	int drop;
 };
 
 /* Most of the time seems to be spent in AviSynth (about 4/5). */
 int main (int argc, char *argv[])
 {
-	struct framerate_entry_s framerates[] = {{"23.976", "23.976", 24, 0}, {"23.976d", "23.976", 24000/1001.0, 1}, {"24", "24", 24, 0}, {"25", "25", 25, 0}, {"29.97", "29.97", 30, 0}, {"29.97d", "29.97", 30000/1001.0, 1}, {"50", "50", 50, 0}, {"59.94", "59.94", 60, 0}, {"59.94d", "59.94", 60000/1001.0, 1}, {NULL, NULL, 0, 0}};
+	struct framerate_entry_s framerates[] = { {"23.976", "23.976", 24, 0}
+	                                        /*, {"23.976d", "23.976", 24000/1001.0, 1}*/
+	                                        , {"24", "24", 24, 0}
+	                                        , {"25", "25", 25, 0}
+	                                        , {"29.97", "29.97", 30, 0}
+	                                        /*, {"29.97d", "29.97", 30000/1001.0, 1}*/
+	                                        , {"50", "50", 50, 0}
+	                                        , {"59.94", "59.94", 60, 0}
+	                                        /*, {"59.94d", "59.94", 60000/1001.0, 1}*/
+	                                        , {NULL, NULL, 0, 0}
+	                                        };
 	char *avs_filename, *track_name, *language, *video_format, *frame_rate, *out_filename;
 	char *in_img, *old_img, *tmp, *out_buf;
 	char *intc_buf, *outtc_buf;
 	char *drop_frame;
-	double fps;
+	int fps;
 	int frames;
 	int first_frame = -1, start_frame, end_frame;
 	int num_of_events = 0;
